#include <SPI.h>


const uint8_t musesChipAddress = 0;   // Chip address must correspond to the state of MUSES72323 pins 30..31

#define MUSES_CS    5  // Chip Select   (SS)


// MUSES72323 register addresses
// Each register is 16 bit
// Bits 15..8 contain the data (volume setting or control bits)
// Bits 7..4 are register address
// Bits 3..0 are chip address, corresponding to the state of MUSES72320 pins 29..31.
const uint8_t musesLeftAtt   = 0;       // 左声道衰减
const uint8_t musesRightAtt  = 0x02<<4; // 右声道衰减（100000）
const uint8_t musesControl   = 0x04<<4; // 控制位（1000000）
const uint8_t musesRegMask   = 0x07<<4; // 寄存器地址掩码（1110000）
const uint8_t musesZcBit     = 0x01<<5; // set bit 5 LOW in the data word and set zcenPin LOW to enable zero crossing detection
const uint8_t musesLRAtt     = 0x01<<7; // set bit 7 HIGH to link right channel attenuation to left channel control register

const long minVolume = -230;      // 203 = -50.5dB
const long maxVolume = -1;        // = 0 dB



// ------------------------------------------------------------------------------------

void setup() {
  Serial.begin(115200);
  pinMode(MUSES_CS, OUTPUT);
  digitalWrite(MUSES_CS, HIGH);  
  SPI.begin();
  SPI.beginTransaction(SPISettings(250000, MSBFIRST, SPI_MODE0));  // MUSES72323 SPI interface is rated at 250kHz maximum
}




// ------------------------------------------------------------------------------------

void loop()
{
  musesWriteVolume(0, 0);
  delay(3000); 
  for (int i=0;i>=-50;i--) {
    musesWriteVolume(i, i);
    delay(50);
  }
  for (int i=-50;i<=0;i++) {
    musesWriteVolume(i, i);
    delay(50);
  }
}



// ------------------------------------------------------------------------------------

uint8_t volumeToAtt(long volume) {
  uint8_t att;
  volume = constrain(volume, minVolume, maxVolume);
  if (0 < volume) {                   // no attenuation 
    att = 0x10;                       // 0dB
    
  } else if (minVolume == volume) {   // mute
    att = 0xFF;                       // Mute, maybe don't do this - handle mute elsewhere

  } else {                            // attenuation from 0 db (volume = 0) to -115.5 dbB (volume == -447)
    volume--;
    volume >>= 1;
    volume = ~volume;
    att = volume & 0xFF; 
    att += 0x10;
  }
  return att;
}


// ------------------------------------------------------------------------------------

void musesWriteVolume(long leftVolume, long rightVolume) {  
  uint8_t address, data;

  address = musesLeftAtt | (musesChipAddress & ~musesRegMask);
  data = volumeToAtt(leftVolume);
  musesWriteRaw(address, data);

  address = musesRightAtt | (musesChipAddress & ~musesRegMask);
  data = volumeToAtt(rightVolume);
  musesWriteRaw(address, data); 
}



// ------------------------------------------------------------------------------------

void musesWriteRaw(uint8_t address, uint8_t data) {
  Serial.print(data,BIN);
  Serial.print("\t");
  Serial.println(address,BIN);

  digitalWrite(MUSES_CS, LOW);
  delayMicroseconds(1);   
  SPI.transfer(data);
  SPI.transfer(address);  
  delayMicroseconds(1);   
  digitalWrite(MUSES_CS, HIGH);
}




// ---------------------- the end ----------------------------
